/***********************************************************
* LCD Interface Library for the Hitachi HD44780 controller *
*           (C) 2009 by Marius Petrescu, YO2LOJ            *
*       Released under the GNU General Public License      *
***********************************************************/

#include <avr/io.h>
#include <avr/pgmspace.h>

#include "lcd.h"

/*****************************************/
/* no architecture dependant parts below */
/*****************************************/

/* start addresses for the lcd lines */
char lcd_line_cmd[2] = {0x00,0x40};

/* delay routine */
void lcd_delay(long int d)
{
	long int i;
	i = d; while(i--) __asm__ __volatile__ ("nop\n nop\n");
}

/* pulse the E line - macro is more efficient than function call */
#define lcd_pulse_e()	{LCD_SET_E(); lcd_delay(10); LCD_CLEAR_E();}

/* send command byte to lcd */
void lcd_sendcmd(char cmd)
{
	LCD_CLEAR_RS();
	LCD_SEND_NIBBLE(cmd >> 4);
	lcd_pulse_e();
	LCD_SEND_NIBBLE(cmd);
	lcd_pulse_e();
	lcd_delay(1000);
}

/* send character to lcd */
void lcd_sendchar(char ch)
{
	LCD_SET_RS();
	LCD_SEND_NIBBLE(ch >> 4);
	lcd_pulse_e();
	LCD_SEND_NIBBLE(ch);
	lcd_pulse_e();
	LCD_CLEAR_RS();
	lcd_delay(1000);
}

/* initialise sequence for the lcd */
void lcd_init(void)
{
	LCD_SET_DATA_WR();
	LCD_SET_CTL_WR();

	lcd_delay(15000);
	LCD_PORT = 0x03;
	lcd_pulse_e();
	lcd_delay(5000);
	LCD_PORT = 0x03;
	lcd_pulse_e();
	lcd_delay(1000);
	LCD_PORT = 0x03;
	lcd_pulse_e();
	lcd_delay(5000);
	LCD_PORT = 0x02;
	lcd_pulse_e();
	lcd_delay(5000);
	
	lcd_sendcmd(0x28);
	lcd_sendcmd(0x08);
	lcd_sendcmd(0x0C);
	lcd_sendcmd(0x06);
	
	lcd_clear();
}

/* clear the lcd */
void lcd_clear(void)
{
	lcd_sendcmd(0x01);
	lcd_delay(5000);
	lcd_sendcmd(0x02);
	lcd_delay(5000);
}

/* PROGMEM enabled output null terminated string to lcd */
void lcd_puts_P(const prog_char *str)
{
	char ch;
	while((ch = pgm_read_byte(str++))) lcd_sendchar(ch);	
}
